﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Malovani
{
    public partial class Window : Form
    {
        public Window()
        {
            InitializeComponent();
            pictureBox_SizeChanged (null, null);
            p = new Pen (panel1.BackColor);
            c = panel2.BackColor;
            b = new SolidBrush(c);
            setup (panel2);
            setup (panel3);
        }

        private void pictureBox_SizeChanged(object sender, EventArgs e)
        {
            Image last = pictureBox.Image;
            int w = pictureBox.Width;
            int h = pictureBox.Height;
            if (w > 0 && h > 0)
            {
               if (last != null)
               {
                   w = Math.Max (w, last.Width);
                   h = Math.Max (h, last.Height);
               }
               pictureBox.Image = new Bitmap (w, h);
               Graphics g = Graphics.FromImage (pictureBox.Image);
               Brush b = new SolidBrush (Color.White);
               g.FillRectangle(b, 0, 0, w, h);
               if (last != null)
                  g.DrawImage (last, 0, 0);
            }
        }

        private int X0, Y0;
        private bool pressed = false;
        private Bitmap save;

        private Pen p;
        private Color c;
        private Brush b;

        private int cnt = 1;

        private void setup (Panel panel)
        {
            panel.Left = panel1.Left * (cnt+1) + 
                         panel1.Width * cnt;
            cnt ++;
            panel.Top = panel1.Top;
            panel.Size = panel1.Size;
        }

        private void newColorItem_Click(object sender, EventArgs e)
        {
            Panel panel = new Panel();
            // panel.Parent = toolPanel;
            toolPanel.Controls.Add (panel);
            panel.BackColor = Color.Yellow;
            setup (panel);
            panel.MouseDown += panel1_MouseDown;
        }

        private void panel1_MouseDown (object sender, MouseEventArgs e)
        {
            Panel panel = (Panel) sender;
            if (Control.ModifierKeys == Keys.Control)
            {
                colorDialog1.Color = panel.BackColor;
                if (colorDialog1.ShowDialog () == DialogResult.OK)
                    panel.BackColor = colorDialog1.Color;

            }
            else if (e.Button == MouseButtons.Left)
            {
                p = new Pen(panel.BackColor);
            }
            else if (e.Button == MouseButtons.Right)
            {
                c = panel.BackColor;
                b = new SolidBrush(c);
            }
            else if (e.Button == MouseButtons.Middle)
            {
                // using System.Drawing.Drawing2D;
                int x = (int)numericUpDown1.Value;
                int y = (int)numericUpDown2.Value;
                b = new LinearGradientBrush
                    (new Point(0, 0),
                     new Point(x, y),
                     panel.BackColor,
                     c);
            }

            /*
            Button button1 = (Button) sender;
            
            Button button2 = null;
            if (sender is Button)
                button2 = (Button) sender;

            Button button3 = sender as Button;
            */
        }

        private void pictureBox_MouseDown (object sender, MouseEventArgs e)
        {
            X0 = e.X;
            Y0 = e.Y;
            pressed = true;
            save = new Bitmap (pictureBox.Image);
        }

        private void pictureBox_MouseMove(object sender, MouseEventArgs e)
        {
            if (pressed)
            {
                Graphics g = Graphics.FromImage (pictureBox.Image);
                g.DrawImage (save, 0, 0);
                if (e.Button == MouseButtons.Left)
                    g.DrawLine(p, X0, Y0, e.X, e.Y);
                else if (e.Button == MouseButtons.Middle)
                {
                    int X1 = X0;
                    int Y1 = Y0;
                    int X2 = e.X;
                    int Y2 = e.Y;
                    if (X1 > X2) { int t = X1; X1 = X2; X2 = t; }
                    if (Y1 > Y2) { int t = Y1; Y1 = Y2; Y2 = t; }
                    g.FillRectangle (b, X1, Y1, X2 - X1, Y2 - Y1);
                    g.DrawRectangle (p, X1, Y1, X2 - X1, Y2 - Y1);
                }
                else if (e.Button == MouseButtons.Right)
                {
                    g.FillEllipse (b, X0, Y0, e.X - X0, e.Y - Y0);
                    g.DrawEllipse (p, X0, Y0, e.X - X0, e.Y - Y0);
                }
                pictureBox.Invalidate();
            }
        }

        private void pictureBox_MouseUp(object sender, MouseEventArgs e)
        {
            pressed = false;
        }

        private void newWindowToolStripMenuItem_Click(object sender, EventArgs e)
        {
        }

        private void newWindowItem_Click(object sender, EventArgs e)
        {
            Window w = new Window();
            w.Show();

        }

        private void openItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {
                pictureBox.Image = new Bitmap (openFileDialog1.FileName);
                pictureBox_SizeChanged (null, null);
                // saveFileDialog1.FileName = openFileDialog1.FileName;
                // using System.IO;
                FileInfo f = new FileInfo (openFileDialog1.FileName);
                saveFileDialog1.InitialDirectory = f.DirectoryName;
                saveFileDialog1.FileName = f.Name;
            }
        }

        private void saveItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                int inx = saveFileDialog1.FilterIndex;
                // using System.Drawing.Imaging;
                ImageFormat fmt = ImageFormat.Bmp;
                if (inx == 2)
                    fmt = ImageFormat.Jpeg;
                pictureBox.Image.Save (saveFileDialog1.FileName, fmt);
                // openFileDialog1.FileName = saveFileDialog1.FileName;
            }

        }

        private void newBoxtem_Click(object sender, EventArgs e)
        {
            Panel box = new Panel ();
            box.BackColor = Color.Yellow;
            pictureBox.Controls.Add (box);

            box.MouseDown += this.box_MouseDown;
            box.MouseMove += box_MouseMove;
            box.MouseUp += box_MouseUp;

        }

        private int XS, YS;
        private bool down = false;

        private void box_MouseDown (object sender, MouseEventArgs e)
        {
            XS = e.X;
            YS = e.Y;
            down = true;

            if (e.Button == MouseButtons.Middle)
            {
                Panel box = (Panel) sender;
                colorDialog1.Color = box.BackColor;
                if (colorDialog1.ShowDialog() == DialogResult.OK)
                    box.BackColor = colorDialog1.Color;
            }
        }

        private void box_MouseMove (object sender, MouseEventArgs e)
        {
            if (down)
            {
                Panel box = (Panel) sender;
                if (e.Button == MouseButtons.Left)
                {
                    box.Left += e.X - XS;
                    box.Top += e.Y - YS;
                }
                if (e.Button == MouseButtons.Right)
                {
                    box.Width += e.X - XS;
                    box.Height += e.Y - YS;
                    XS = e.X;
                    YS = e.Y;
                }
            }
        }
        private void box_MouseUp(object sender, MouseEventArgs e)
        {
            down = false;
        }
    }
}
